package com.fs.course.mapper;

import java.util.List;
import com.fs.course.domain.FsCourseTrafficLog;
import com.fs.course.param.FsCourseTrafficLogParam;
import com.fs.course.param.TrafficRecord;
import com.fs.course.vo.FsCourseTrafficLogListVO;
import com.fs.statis.dto.TrafficLogDTO;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;

/**
 * 短链课程流量记录Mapper接口
 *
 * @author fs
 * @date 2024-10-31
 */
public interface FsCourseTrafficLogMapper
{
    /**
     * 查询短链课程流量记录
     *
     * @param logId 短链课程流量记录主键
     * @return 短链课程流量记录
     */
    public FsCourseTrafficLog selectFsCourseTrafficLogByLogId(Long logId);

    /**
     * 查询短链课程流量记录列表
     *
     * @param fsCourseTrafficLog 短链课程流量记录
     * @return 短链课程流量记录集合
     */
    public List<FsCourseTrafficLog> selectFsCourseTrafficLogList(FsCourseTrafficLog fsCourseTrafficLog);

    /**
     * 新增短链课程流量记录
     *
     * @param fsCourseTrafficLog 短链课程流量记录
     * @return 结果
     */
    public int insertFsCourseTrafficLog(FsCourseTrafficLog fsCourseTrafficLog);

    /**
     * 修改短链课程流量记录
     *
     * @param fsCourseTrafficLog 短链课程流量记录
     * @return 结果
     */
    public int updateFsCourseTrafficLog(FsCourseTrafficLog fsCourseTrafficLog);

    /**
     * 删除短链课程流量记录
     *
     * @param logId 短链课程流量记录主键
     * @return 结果
     */
    public int deleteFsCourseTrafficLogByLogId(Long logId);

    /**
     * 批量删除短链课程流量记录
     *
     * @param logIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteFsCourseTrafficLogByLogIds(Long[] logIds);


    List<FsCourseTrafficLogListVO> selectTrafficByCompany(@Param("maps") FsCourseTrafficLogParam param);

    @Select("select * from fs_course_traffic_log where uu_id = #{uuId}")
    FsCourseTrafficLog selectFsCourseTrafficLogByuuId(@Param("uuId") String uuId);

    void insertOrUpdateTrafficLog(FsCourseTrafficLog trafficLog);

    List<FsCourseTrafficLogListVO> selectTrafficNew(FsCourseTrafficLogParam param);


    /**
     * 获取当天统计流量
     * @return Long
     */
    Long getTodayTrafficLog();
    Long getTodayTrafficLogCompanyId(@Param("companyId") Long companyId,@Param("companyUserId") Long companyUserId);

    /**
     * 获取当月统计流量
     * @return Long
     */
    Long getMonthTrafficLog();
    Long getMonthTrafficLogCompanyId(@Param("companyId") Long companyId,@Param("companyUserId") Long companyUserId);

    /**
     * 昨天
     * @return Long
     */
    Long getYesterdayTrafficLog();
    Long getYesterdayTrafficLogCompanyId(@Param("companyId") Long companyId,@Param("companyUserId") Long companyUserId);



    // 按创建时间查询未使用的流量记录（用于分批更新）
    @Select("<script>" +
            "SELECT COALESCE(sum(internet_traffic), 0) FROM fs_course_traffic_log " +
            "WHERE status = 0" +
            "<if test='companyId != null'>AND company_id = #{companyId}</if> " +
            "</script>")
    Long findRecordsNum(@Param("companyId") Long companyId);

    // 批量更新状态
    @Update("<script>" +
            "UPDATE fs_course_traffic_log SET status = 1 WHERE log_id IN " +
            "<foreach item='id' collection='ids' open='(' separator=',' close=')'>" +
            "#{id}" +
            "</foreach>" +
            "</script>")
    int updateStatusByIds(@Param("ids") List<Long> ids);

    @Select("<script>" +
            "SELECT internet_traffic FROM fs_course_traffic_log WHERE log_id IN " +
            "<foreach item='id' collection='ids' open='(' separator=',' close=')'>" +
            "#{id}" +
            "</foreach>" +
            "</script>")
    List<Long> getTrafficByIds(@Param("ids") List<Long> ids);

    // 新增带流量字段的记录查询（按公司ID）
    @Select("<script>" +
            "SELECT log_id, internet_traffic FROM fs_course_traffic_log " +
            "WHERE status = 0" +
            "<if test='companyId != null'>AND company_id = #{companyId}</if> " +
            "ORDER BY create_time ASC " +
            "LIMIT #{offset}, #{pageSize}" +
            "</script>")
    List<TrafficRecord> findUnusedRecordsWithTraffic(
            @Param("companyId") Long companyId,
            @Param("offset") int offset,
            @Param("pageSize") int pageSize);


}
